#include "videokit.h"
#include "videoplayertool.h"
#include "AgoraBase.h"
#include <Windows.h>
#include <memory>
#include <thread>
#include <iostream>
using namespace agora::media;
using namespace agora::rtc;

static void videoRawDataCallBack(int linsize[], void *data, int w, int h) {

	VideoKit::Get()->onVideoDataCallBack(linsize, data, w, h);
}
static void audioRawDataCallBack(void *data, int len) {
	VideoKit::Get()->onAudioDataCallBack(data, len);
}
static void audioInfoCallBack(int index, int sample_rate, int channles, long duation) {

	VideoKit::Get()->onAudioInfoCallBack(index, sample_rate, channles, duation);

}
static void videoStateCallBack(VIDEO_STATE state) {
	VideoKit::Get()->onVideoStateCallBack((FF_VIDEO_STATE)state);

}
static void videoInfoCallBack(int index, int width, int height) {
	VideoKit::Get()->onVideoInfoCallBack(index, width, height);
}
void VideoKit::onVideoDataCallBack(int linesize[], void *data, int w, int h) {

	if (isExit || !_rtcEngine)
	{
		return;
	}

	int size = w * h * 3 / 2;
	BYTE *tmp = (BYTE *)malloc(size);
	memset(tmp, 0, size);
	if (linesize[0] != w)
	{
		for (int i = 0; i < h; i++)
		{
			memcpy(tmp + i * w, (BYTE *)data + i * linesize[0], w);
		}
		for (int i = 0; i < h / 2; i++)
		{
			memcpy(tmp + w * h + w / 2 * i, (BYTE *)data + linesize[0] * h + i * linesize[0] / 2, w / 2);

		}
		for (int i = 0; i < h / 2; i++)
		{
			memcpy(tmp + w * h * 5 / 4 + w / 2 * i, (BYTE *)data + linesize[0] * h * 5 / 4 + linesize[0] / 2 * i, w / 2);

		}
	}
	else {
		memcpy(tmp, data, size);

	}
	if (!initRtcEngine || !isVieoPublish)
	{
		free(tmp);
		return;
	}
	ExternalVideoFrame frame;
	frame.stride = w;
	frame.height = h;
	frame.timestamp = static_cast<long long>(GetHighAccuracyTickCount());
	frame.rotation = 0;
	frame.type = ExternalVideoFrame::VIDEO_BUFFER_TYPE::VIDEO_BUFFER_RAW_DATA;
	frame.format = ExternalVideoFrame::VIDEO_PIXEL_FORMAT::VIDEO_PIXEL_I420;
	frame.cropLeft = 0;
	frame.cropTop = 0;
	frame.cropBottom = 0;
	frame.cropRight = 0;
	frame.buffer = tmp;
	agora::util::AutoPtr<agora::media::IMediaEngine> mediaEngine;
	mediaEngine.queryInterface(_rtcEngine, agora::AGORA_IID_MEDIA_ENGINE);

	if (mediaEngine)
		mediaEngine->pushVideoFrame(&frame);

	free(tmp);
}

void VideoKit::onAudioDataCallBack(void *data, int len) {

	if (!isAudioPublish || !initRtcEngine || !_rtcEngine) {
		return;
	}
	m_mutex.lock();
	CExtendAudioFrameObserver::Get()->pushAudioData(data, len);
	m_mutex.unlock();

}

void VideoKit::onAudioInfoCallBack(int index, int sample_rate, int channels, long duration) {


	CExtendAudioFrameObserver::Get()->re_channels = channels;
	CExtendAudioFrameObserver::Get()->re_sample_rate = sample_rate;
	this->duration = duration;
	audio_chs = channels;
	if (mediaInfoCallback != NULL)
	{
                mediaInfoCallback->onAudioTrackInfoCallback(index, sample_rate, channels, duration);
	}
}
void VideoKit::onVideoInfoCallBack(int index, int width, int height) {
	pic_width = width;
	pic_height = height;

	if (mediaInfoCallback != NULL)
	{
		mediaInfoCallback->onVideoTrackInfoCallback(index, width, height);

	}

}
void VideoKit::onVideoStateCallBack(FF_VIDEO_STATE state) {

	if (!this->eventHandler)
	{
		return;
	}

	switch (state)
	{
	case FF_VIDEO_START:
	{
		break;
	}
	case FF_VIDEO_ERROR:

		break;
	case FF_VIDEO_EXIT:
		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_STOP);
		break;
	case FF_VIDEO_COMPLETE:
	{
		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_COMPLETE);
		break;
	}
	case FF_VIDEO_PREPARE:
	{

		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_READY);

		break;
	}
	case FF_VIDEO_PAUSE:
	{

		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_PAUSE);

		break;

	}
	case FF_VIDEO_PLAY:
	{

		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_PLAY);

		break;
	}
	case FF_VIDEO_FILE_ERROR:
	{
		isPlaying = false;
		isExit = true;
		duration = 0;
		this->eventHandler->onKitError(AG_SOURCE_LOADED_ERROR_UNSUPPORTED_FORMAT);
		break;
	}
	case FF_VIDEO_INVALID_ARGUMENT:
	{
		isPlaying = false;
		isExit = true;
		duration = 0;
		this->eventHandler->onKitError(AG_SOURCE_LOADED_ERROR_UNSUPPORTED_ARGUMENT);

		break;
	}
	case FF_VIDEO_UNSUPPORT_FORMATE:
	{
		isPlaying = false;
		isExit = true;
		duration = 0;
		this->eventHandler->onKitError(AG_SOURCE_LOADED_ERROR_UNSUPPORTED_FORMAT);
		unload();
		break;
	}
	case FF_VIDEO_NETWORK_UNVAILABLE:
	{
		isPlaying = false;
		isExit = true;
		duration = 0;
		this->eventHandler->onKitError(AG_PLAY_ERROR_NETWORK_UNAVAILABLE);

		break;
	}
	case FF_VIDEO_STOP: {

		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_STOP);

		break;
	}
	case FF_VIDEO_BUFFERING: {


		this->eventHandler->onPlayerStateChanged(AG_MEDIA_PLAYER_STATE_BUFFERING);

		break;
	}
	case FF_VIDEO_CORRUPTED_DATA: {

		isPlaying = false;
		isExit = true;
		duration = 0;
		this->eventHandler->onKitError(AG_SOURCE_LOADED_ERROR_CORRUPTED_DATA);

	}
	default:
		break;
	}





}
void VideoKit::CreateVideoPlayerKitWithRtcEngine(agora::rtc::IRtcEngine *rtcEngine, int sampleRate) {

	//enableVideoSource
	m_mutex.lock();
	initRtcEngine = true;
	CExtendAudioFrameObserver::Get()->sample_rate = sampleRate;
	_rtcEngine = rtcEngine;
	agora::util::AutoPtr<agora::media::IMediaEngine> mediaEngine;
	mediaEngine.queryInterface(rtcEngine, agora::AGORA_IID_MEDIA_ENGINE);
	if (mediaEngine)
		mediaEngine->registerAudioFrameObserver(CExtendAudioFrameObserver::Get());

	AParameter apm(rtcEngine);
	apm->setParameters("{\"che.video.local.camera_index\":1025}");
	if (mediaEngine)
		mediaEngine->setExternalVideoSource(true, false);
	// set samplerate chanels
	RtcEngineParameters rep(rtcEngine);
	// rep.setExternalAudioSource(false,sampleRate,2);
	rep.setRecordingAudioFrameParameters(sampleRate, 2, RAW_AUDIO_FRAME_OP_MODE_READ_WRITE, sampleRate * 0.01 * 2);
	rep.setPlaybackAudioFrameParameters(sampleRate, 2, RAW_AUDIO_FRAME_OP_MODE_READ_WRITE, sampleRate * 0.01 * 2);
	m_mutex.unlock();

}
typedef struct MyData
{
	char *val1;
	int  val2;
	//char key[32];
	VideoPlayerTool *videoTool;
}MYDATA;

void *ThreadProc(const MyData *mydata)
{
	VideoPlayerTool *videoTool = mydata->videoTool;
	videoTool->setVideoCallBack(videoRawDataCallBack);
	videoTool->setAudioCallBack(audioRawDataCallBack);
	videoTool->setAudioInfoCallBack(audioInfoCallBack);
	videoTool->setVideoInfoCallBack(videoInfoCallBack);
	videoTool->setVideoStateCallBack(videoStateCallBack);

	std::string path = mydata->val1;
	char *videopath = (char *)path.data();
	videoTool->load_video(videopath, mydata->val2);
	delete videoTool;
	return nullptr;
}
static MYDATA mydt;
static char * videoPath = NULL;
static bool isAuto_play = false;
static long time_join = 0;
static long time_leave = 0;
static long time_play = 0;
static long time_duration = 0;
void VideoKit::load(char * source, bool auto_play) {
	if (isPlaying)
		return;
	isPlaying = true;
	isExit = false;
	isUnload = false;
	//windows߳
	HANDLE hThread;
	mydt.val1 = source;
	mydt.val2 = auto_play;
	time_join = static_cast<long long>(VideoKit::Get()->GetHighAccuracyTickCount());
	mydt.videoTool = new VideoPlayerTool();
	std::thread t(ThreadProc, &mydt);
	t.detach();
	Sleep(100);
}
void VideoKit::play() {

	m_mutex.lock();
	time_play = static_cast<long long>(VideoKit::Get()->GetHighAccuracyTickCount());
	if ((time_play - time_join) < 600)
	{
		Sleep(600);
	}
	CExtendAudioFrameObserver::Get()->reset();
	videoTool->video_seek(0);
	m_mutex.unlock();
}
void VideoKit::pause() {

	m_mutex.lock();
	videoTool->video_play_pause();
	CExtendAudioFrameObserver::Get()->reset();
	m_mutex.unlock();

}
void VideoKit::resume() {
	m_mutex.lock();
	videoTool->video_play_pause();
	CExtendAudioFrameObserver::Get()->reset();
	m_mutex.unlock();
}
void VideoKit::unload() {

	m_mutex.lock();
	isPlaying = false;
	isExit = true;
	duration = 0;
	isUnload = true;
	time_leave = static_cast<long long>(VideoKit::Get()->GetHighAccuracyTickCount());
	long time = time_leave - time_join;
	if (time < 800);
	{
		Sleep(800);
	}
	printf("timestamp ==  %ld", time);
	videoTool->video_quit();
	Sleep(400);
	m_mutex.unlock();

}
long VideoKit::getCurrentPosition() {
	if (this->duration == 0)
	{
		return 0;
	}
	m_mutex.lock();
	double ret = videoTool->getVideoCurrentPos();
	m_mutex.unlock();
	return ret;
}
void VideoKit::seekTo(long msec) {

	m_mutex.lock();
	CExtendAudioFrameObserver::Get()->reset();
	videoTool->video_seek(msec);
	m_mutex.unlock();

}

void VideoKit::setHandler(VideoPlayerKitEventHandler *handler) {
	m_mutex.lock();
	this->handler = handler;
	m_mutex.unlock();
}
void VideoKit::setAudioMixing(bool isAudioMix) {
	m_mutex.lock();
	CExtendAudioFrameObserver::Get()->setAudioMixing(isAudioMix);
	m_mutex.unlock();
}
void VideoKit::publish() {
	m_mutex.lock();
	isVieoPublish = false;
	m_mutex.unlock();

}
void VideoKit::unpublish
() {
	m_mutex.lock();
	isVieoPublish = true;
	m_mutex.unlock();
}
void  VideoKit::setVideoView(void *hwnd) {

	m_mutex.lock();
	videoTool->setWinID(hwnd);
	m_mutex.unlock();

}

long VideoKit::getDuration() {
	time_duration = static_cast<long long>(VideoKit::Get()->GetHighAccuracyTickCount());
	if (time_duration - time_join < 600)
	{
		Sleep(600);
	}
	return this->duration;
}
void VideoKit::adjustPlaybackSignalVolume(int volume)
{
	m_mutex.lock();
	videoTool->setPlaySignalVolume(volume);
	m_mutex.unlock();
}
void VideoKit::stop() {
	m_mutex.lock();
	videoTool->video_stop();
	m_mutex.unlock();
}
void VideoKit::destroy() {
	isPlaying = false;
	isExit = true;
	duration = 0;

	if (!isUnload)
	{
		unload();
	}
	m_mutex.lock();
	if (_rtcEngine)
	{
		agora::util::AutoPtr<agora::media::IMediaEngine> mediaEngine;
		mediaEngine.queryInterface(_rtcEngine, agora::AGORA_IID_MEDIA_ENGINE);
		if (mediaEngine)
			mediaEngine->registerAudioFrameObserver(NULL);
	}
	Sleep(200);
	_rtcEngine = NULL;
	m_mutex.unlock();
}




void VideoKit::publishAudio() {
	m_mutex.lock();
	if (audio_chs < 0 || audio_chs > 2)
	{
		this->eventHandler->onKitError(AG_SOURCE_LOADED_ERROR_NO_AVAILABLE_TRACK);
		m_mutex.unlock();
		return;
	}
	isAudioPublish = true;
	m_mutex.unlock();
}
void VideoKit::publishVideo() {
	m_mutex.lock();
	isVieoPublish = true;
	m_mutex.unlock();
}
void VideoKit::unpublishAudio() {
	m_mutex.lock();
	isAudioPublish = false;
	m_mutex.unlock();
}
void VideoKit::unpublishVideo() {
	m_mutex.lock();
	isVieoPublish = false;
	m_mutex.unlock();
}
void VideoKit::adjustPublishSignalVolume(int volume) {
	m_mutex.lock();
	CExtendAudioFrameObserver::Get()->setRemoteVolume(volume);
	m_mutex.unlock();
}
void VideoKit::setEventHandler(MediaPlayerKitEventHandler *handler) {
	m_mutex.lock();
	eventHandler = handler;
	m_mutex.unlock();
}
void VideoKit::setMediaInfoCallback(MediaInfoCallback *callback) {
	m_mutex.lock();
	mediaInfoCallback = callback;
	m_mutex.unlock();
}
MediaPlayerKit * createMediaPlayerKit() {
	return VideoKit::Get();
}
MediaPlayerKit * createMediaPlayerKitWithRtcEngine(void * engine, int sample_rate) {
	VideoKit::Get()->CreateVideoPlayerKitWithRtcEngine((agora::rtc::IRtcEngine *)engine, sample_rate);
	return VideoKit::Get();
}
