#ifndef VIDEOKIT_H
#define VIDEOKIT_H
#include "videoinfocallback.h"
#include <IAgoraMediaEngine.h>
#include <IAgoraRtcEngine.h>
#include "ExtendAudioFrameObserver.h"
#include <memory>
#include <mutex>
#include "SingleTon.h"
#include "MediaPlayerKit.h"
class VideoPlayerTool;

enum FF_PLAYER_STATE
{
	FF_PLAYER_STATE_PLAY = 0,
	FF_PLAYER_STATE_PAUSE = 1
	
};
enum FF_LOAD_VIDEO_STATE
{
	FF_LOAD_VIDEO_ERROR_UNSORPPORT_FORMATE = 0, 
	FF_LOAD_VIDEO_ERROR_NULL_FILE = 1,
	FF_LOAD_VIDEO_ERROR_UNSORPPORT_ARGUMENT = 2,
	FF_LOAD_VIDEO_ERROR_PREPARED = 3,
};

class VideoPlayerKitEventHandler{
public:
	VideoPlayerKitEventHandler(){}
	virtual ~VideoPlayerKitEventHandler(){

	}
	//回调函数
	virtual void onVideoLoaded(FF_LOAD_VIDEO_STATE state){};
	//执行失败的回调
	virtual void onKitError(){};
	//播放器停止的回调
	virtual void onPlayerStopped(){};
	//播放状态的回调
	virtual void onPlayerSate(FF_PLAYER_STATE playerState){};
};

class VideoKit : public VideoInfoCallBack,public Singleton<VideoKit>,public MediaPlayerKit
{
	friend Singleton;
public:
	~VideoKit() {
	
	}
	VideoKit()
	{
		duration = 0;
		handler = NULL;
		initRtcEngine = false;
	}
	long duration;
	virtual void onVideoDataCallBack(int linesize[], void *data, int w, int h);
	virtual void onAudioDataCallBack(void *data, int len);
	virtual void onAudioInfoCallBack(int index,int sample_rate, int channles, long duation);
	virtual void onVideoInfoCallBack(int index, int width, int height);
	virtual void onVideoStateCallBack(FF_VIDEO_STATE state);
    //需要注册videoplaykit的引擎
    void CreateVideoPlayerKitWithRtcEngine(agora::rtc::IRtcEngine *rtcEngine, int sampleRate);
    //设置接口回调
	void setHandler(VideoPlayerKitEventHandler *handler);
    //打开需要播放的文件
    virtual void load(char * source,bool auto_play);
    //获取当前文件播放的进度
	virtual long getCurrentPosition();
    //跳转到视频文件播放的位置
	virtual void seekTo(long msec);
    //从头开始
	virtual void play();
    //暂停播放
	virtual void pause();
    //恢复播放
	virtual void resume();
    //退出视频播放
    void unload();
    //是否混入麦克风的声音
	virtual void setAudioMixing(bool isAudioMix);
	//开启视频预览
	virtual void publish();
	//如果之前有开启视频预览请务必在加入频道成功之后 调用该接口关闭视频预览  
	virtual void unpublish();
	//设置需要渲染的窗口句柄
	virtual void setVideoView(void * hwnd);
	virtual void setEventHandler(MediaPlayerKitEventHandler *handler);
	virtual void setMediaInfoCallback(MediaInfoCallback *callback);
	virtual long getDuration();
	virtual void stop();
	virtual void destroy();
	virtual void publishAudio();
	virtual void unpublishAudio();
	virtual void publishVideo();
	virtual void unpublishVideo();
	virtual void adjustPublishSignalVolume(int volume);
	virtual void adjustPlaybackSignalVolume(int volume);
	//timeStamp
	static UINT64 WINAPI GetHighAccuracyTickCount()
	{
		LARGE_INTEGER	liPC;
		LARGE_INTEGER	liPF;

		::QueryPerformanceCounter(&liPC);
		::QueryPerformanceFrequency(&liPF);

		return static_cast<UINT64>(liPC.QuadPart * 1000 / liPF.QuadPart);
	}


private:
	bool isExit = false;
	bool isPlaying = false;
	bool isVieoPublish = false;
	bool isAudioPublish = false;
	int audio_chs = 0;
	bool isUnload = false;
    agora::rtc::IRtcEngine *_rtcEngine;
	VideoPlayerTool *videoTool;
    std::mutex m_mutex;
	VideoPlayerKitEventHandler *handler;
	bool initRtcEngine;
	int m_width;
	int m_height;
	void *m_hwnd;
	int pic_width;
	int pic_height;
	MediaInfoCallback *mediaInfoCallback = NULL;
	MediaPlayerKitEventHandler * eventHandler = NULL;
};
#endif // VIDEOKIT_H
